@@ -61,25 +61,21 @@ class BleiCorpus(IndexedCorpus):
             else:
                 raise IOError('BleiCorpus: could not find vocabulary file')
 
-
         self.fname = fname
         with utils.smart_open(fname_vocab) as fin:
             words = [utils.to_unicode(word).rstrip() for word in fin]
         self.id2word = dict(enumerate(words))
-        self.length = None
-
+        self.length = 0
 
     def __iter__(self):
         """
         Iterate over the corpus, returning one sparse vector at a time.
         """
-        length = 0
+        lineno = -1
         with utils.smart_open(self.fname) as fin:
             for lineno, line in enumerate(fin):
-                length += 1
                 yield self.line2doc(line)
-        self.length = length
-
+        self.length = lineno + 1
 
     def line2doc(self, line):
         parts = utils.to_unicode(line).split()
@@ -89,7 +85,6 @@ class BleiCorpus(IndexedCorpus):
         doc = [(int(p1), float(p2)) for p1, p2 in doc]
         return doc
 
-
     @staticmethod
     def save_corpus(fname, corpus, id2word=None, metadata=False):
         """
@@ -133,5 +128,5 @@ class BleiCorpus(IndexedCorpus):
         with utils.smart_open(self.fname) as f:
             f.seek(offset)
             return self.line2doc(f.readline())
-#endclass BleiCorpus
 
+# endclass BleiCorpus
