@@ -307,7 +307,7 @@ class Sparse2Corpus(object):
 
     def __iter__(self):
         for indprev, indnow in izip(self.sparse.indptr, self.sparse.indptr[1:]):
-            yield zip(self.sparse.indices[indprev:indnow], self.sparse.data[indprev:indnow])
+            yield list(zip(self.sparse.indices[indprev:indnow], self.sparse.data[indprev:indnow]))
 
     def __len__(self):
         return self.sparse.shape[1]
@@ -582,10 +582,14 @@ class MmReader(object):
         logger.info("initializing corpus reader from %s" % input)
         self.input, self.transposed = input, transposed
         with utils.file_or_filename(self.input) as lines:
-            header = utils.to_unicode(next(lines)).strip()
-            if not header.lower().startswith('%%matrixmarket matrix coordinate real general'):
-                raise ValueError("File %s not in Matrix Market format with coordinate real general; instead found: \n%s" %
-                                 (self.input, header))
+            try:
+                header = utils.to_unicode(next(lines)).strip()
+                if not header.lower().startswith('%%matrixmarket matrix coordinate real general'):
+                    raise ValueError("File %s not in Matrix Market format with coordinate real general; instead found: \n%s" %
+                                    (self.input, header))
+            except StopIteration:
+                pass
+
             self.num_docs = self.num_terms = self.num_nnz = 0
             for lineno, line in enumerate(lines):
                 line = utils.to_unicode(line)
