@@ -16,8 +16,11 @@ Example: python -m gensim.models.lsi_dispatcher
 
 from __future__ import with_statement
 import os, sys, logging, threading, time
-from Queue import Queue
-
+try:
+    from Queue import Queue
+except ImportError:
+    from queue import Queue
+import Pyro4
 from gensim import utils
 
 
@@ -67,16 +70,12 @@ class Dispatcher(object):
         # locate all available workers and store their proxies, for subsequent RMI calls
         self.workers = {}
         with utils.getNS() as ns:
-            import Pyro4
             self.callback = Pyro4.Proxy('PYRONAME:gensim.lsi_dispatcher') # = self
-            self.callback._pyroOneway.add("jobdone") # make sure workers transfer control back to dispatcher asynchronously
             for name, uri in ns.list(prefix='gensim.lsi_worker').iteritems():
                 try:
                     worker = Pyro4.Proxy(uri)
                     workerid = len(self.workers)
                     # make time consuming methods work asynchronously
-                    worker._pyroOneway.add("requestjob")
-                    worker._pyroOneway.add("exit")
                     logger.info("registering worker #%i from %s" % (workerid, uri))
                     worker.initialize(workerid, dispatcher=self.callback, **model_params)
                     self.workers[workerid] = worker
@@ -142,7 +141,7 @@ class Dispatcher(object):
         self._jobsdone = 0
         self._jobsreceived = 0
 
-
+    @Pyro4.oneway
     @utils.synchronous('lock_update')
     def jobdone(self, workerid):
         """
@@ -163,6 +162,7 @@ class Dispatcher(object):
         return self._jobsdone
 
 
+    @Pyro4.oneway
     def exit(self):
         """
         Terminate all registered workers and then the dispatcher.
