@@ -20,8 +20,11 @@ from __future__ import with_statement
 import os, sys, logging
 import threading
 import tempfile
-import Queue
-
+try:
+    import Queue
+except ImportError:
+    import queue as Queue
+import Pyro4
 from gensim.models import lsimodel
 from gensim import utils
 
@@ -47,6 +50,7 @@ class Worker(object):
         self.model = lsimodel.LsiModel(**model_params)
 
 
+    @Pyro4.oneway
     def requestjob(self):
         """
         Request jobs from the dispatcher, in a perpetual loop until `getstate()` is called.
@@ -94,6 +98,7 @@ class Worker(object):
         self.finished = False
 
 
+    @Pyro4.oneway
     def exit(self):
         logger.info("terminating worker #%i" % self.myid)
         os._exit(0)
