@@ -16,7 +16,7 @@ import itertools
 
 from gensim.utils import to_unicode
 from gensim.corpora import (bleicorpus, mmcorpus, lowcorpus, svmlightcorpus,
-                            ucicorpus, malletcorpus, textcorpus)
+                            ucicorpus, malletcorpus, textcorpus, indexedcorpus)
 
 # needed because sample data files are located in the same folder
 module_path = os.path.dirname(__file__)
@@ -122,8 +122,28 @@ class CorpusTestCase(unittest.TestCase):
             testdoc2 = set((to_unicode(corpus.id2word[x]), y) for x, y in firstdoc2)
             self.assertEqual(testdoc2, set([('computer', 1), ('human', 1), ('interface', 1)]))
 
+    def test_indexing(self):
+        fname = datapath('testcorpus.' + self.file_extension.lstrip('.'))
+        corpus = self.corpus_class(fname)
+        docs = list(corpus)
 
-# endclass CorpusTestCase
+        for idx, doc in enumerate(docs):
+            self.assertEqual(doc, corpus[idx])
+
+        self.assertEqual(docs, list(corpus[:]))
+        self.assertEqual(docs[0:], list(corpus[0:]))
+        self.assertEqual(docs[0:-1], list(corpus[0:-1]))
+        self.assertEqual(docs[2:4], list(corpus[2:4]))
+        self.assertEqual(docs[::2], list(corpus[::2]))
+        self.assertEqual(docs[::-1], list(corpus[::-1]))
+
+        # make sure sliced corpora can be iterated over multiple times
+        c = corpus[:]
+        self.assertEqual(docs, list(c))
+        self.assertEqual(docs, list(c))
+        self.assertEqual(len(docs), len(corpus))
+        self.assertEqual(len(docs), len(corpus[:]))
+        self.assertEqual(len(docs[::2]), len(corpus[::2]))
 
 
 class TestMmCorpus(CorpusTestCase):
@@ -135,24 +155,18 @@ class TestMmCorpus(CorpusTestCase):
         # MmCorpus needs file write with seek => doesn't support compressed output (only input)
         pass
 
-# endclass TestMmCorpus
-
 
 class TestSvmLightCorpus(CorpusTestCase):
     def setUp(self):
         self.corpus_class = svmlightcorpus.SvmLightCorpus
         self.file_extension = '.svmlight'
 
-# endclass TestSvmLightCorpus
-
 
 class TestBleiCorpus(CorpusTestCase):
     def setUp(self):
         self.corpus_class = bleicorpus.BleiCorpus
         self.file_extension = '.blei'
 
-# endclass TestBleiCorpus
-
 
 class TestLowCorpus(CorpusTestCase):
     TEST_CORPUS = [[(1, 1)], [], [(0, 2), (2, 1)], []]
@@ -161,8 +175,6 @@ class TestLowCorpus(CorpusTestCase):
         self.corpus_class = lowcorpus.LowCorpus
         self.file_extension = '.low'
 
-# endclass TestLowCorpus
-
 
 class TestUciCorpus(CorpusTestCase):
     TEST_CORPUS = [[(1, 1)], [], [(0, 2), (2, 1)], []]
@@ -175,8 +187,6 @@ class TestUciCorpus(CorpusTestCase):
         # UciCorpus needs file write with seek => doesn't support compressed output (only input)
         pass
 
-# endclass TestUciCorpus
-
 
 class TestMalletCorpus(CorpusTestCase):
     TEST_CORPUS = [[(1, 1)], [], [(0, 2), (2, 1)], []]
@@ -199,8 +209,6 @@ class TestMalletCorpus(CorpusTestCase):
             self.assertEqual(metadata[0], str(i + 1))
             self.assertEqual(metadata[1], 'en')
 
-# endclass TestMalletCorpus
-
 
 class TestTextCorpus(CorpusTestCase):
 
@@ -230,7 +238,8 @@ class TestTextCorpus(CorpusTestCase):
     def test_serialize_compressed(self):
         pass
 
-# endclass TestTextCorpus
+    def test_indexing(self):
+        pass
 
 
 if __name__ == '__main__':
