@@ -19,7 +19,7 @@ import numpy
 import scipy.linalg
 
 from gensim.corpora import mmcorpus, Dictionary
-from gensim.models import lsimodel, ldamodel, tfidfmodel, rpmodel, logentropy_model, ldamallet
+from gensim.models import lsimodel, ldamodel, tfidfmodel, rpmodel, logentropy_model, ldamallet, ldamulticore
 from gensim import matutils
 
 
@@ -209,6 +209,40 @@ class TestLdaModel(unittest.TestCase):
                             (i, sorted(vec), sorted(expected)))
         self.assertTrue(passed)
 
+    def testTopicSeeding(self):
+        passed = False
+        for topic in range(2):
+            # try seeding it both ways round, check you get the same
+            # topics out but with which way round they are depending
+            # on the way round they're seeded
+            for i in range(5): # restart at most 5 times
+
+                eta = numpy.ones((2, len(dictionary))) * 0.5
+                system = dictionary.token2id[u'system']
+                trees = dictionary.token2id[u'trees']
+
+                # aggressively seed the word 'system', in one of the
+                # two topics, 10 times higher than the other words
+                eta[topic, system] *= 10
+
+                model = ldamodel.LdaModel(id2word=dictionary, num_topics=2, passes=200, eta=eta)
+                model.update(corpus)
+
+                topics = [dict((word, p) for p, word in model.show_topic(j)) for j in range(2)]
+
+                # check that the word system in the topic we seeded, got a high weight,
+                # and the word 'trees' (the main word in the other topic) a low weight --
+                # and vice versa for the other topic (which we didn't seed with 'system')
+                result = [[topics[topic].get(u'system',0), topics[topic].get(u'trees',0)],
+                          [topics[1-topic].get(u'system',0), topics[1-topic].get(u'trees',0)]]
+                expected = [[0.385, 0.022],
+                            [0.025, 0.157]]
+                passed = numpy.allclose(result, expected, atol=1e-2)
+                if passed:
+                    break
+                logging.warning("LDA failed to converge on attempt %i (got %s, expected %s)" %
+                                (i, result, expected))
+            self.assertTrue(passed)
 
     def testPersistence(self):
         model = ldamodel.LdaModel(self.corpus, num_topics=2)
@@ -240,6 +274,97 @@ class TestLdaModel(unittest.TestCase):
 #endclass TestLdaModel
 
 
+class TestLdaMulticore(unittest.TestCase):
+    def setUp(self):
+        self.corpus = mmcorpus.MmCorpus(datapath('testcorpus.mm'))
+
+    def testTransform(self):
+        passed = False
+        # sometimes, LDA training gets stuck at a local minimum
+        # in that case try re-training the model from scratch, hoping for a
+        # better random initialization
+        for i in range(5): # restart at most 5 times
+            # create the transformation model
+            model = ldamulticore.LdaMulticore(id2word=dictionary, num_topics=2, passes=100)
+            model.update(corpus)
+
+            # transform one document
+            doc = list(corpus)[0]
+            transformed = model[doc]
+
+            vec = matutils.sparse2full(transformed, 2) # convert to dense vector, for easier equality tests
+            expected = [0.13, 0.87]
+            passed = numpy.allclose(sorted(vec), sorted(expected), atol=1e-2) # must contain the same values, up to re-ordering
+            if passed:
+                break
+            logging.warning("LDA failed to converge on attempt %i (got %s, expected %s)" %
+                            (i, sorted(vec), sorted(expected)))
+        self.assertTrue(passed)
+
+    def testTopicSeeding(self):
+        passed = False
+        for topic in range(2):
+            # try seeding it both ways round, check you get the same
+            # topics out but with which way round they are depending
+            # on the way round they're seeded
+            for i in range(5): # restart at most 5 times
+
+                eta = numpy.ones((2, len(dictionary))) * 0.5
+                system = dictionary.token2id[u'system']
+
+                # aggressively seed the word 'system', in one of the
+                # two topics, 10 times higher than the other words
+                eta[topic, system] *= 10
+
+                model = ldamulticore.LdaMulticore(id2word=dictionary, num_topics=2, passes=200, eta=eta)
+                model.update(corpus)
+
+                topics = [dict((word, p) for p, word in model.show_topic(j)) for j in range(2)]
+
+                # check that the word system in the topic we seeded, got a high weight,
+                # and the word 'trees' (the main word in the other topic) a low weight --
+                # and vice versa for the other topic (which we didn't seed with 'system')
+                result = [[topics[topic].get(u'system',0), topics[topic].get(u'trees',0)],
+                          [topics[1-topic].get(u'system',0), topics[1-topic].get(u'trees',0)]]
+                expected = [[0.385, 0.022],
+                            [0.025, 0.157]]
+                passed = numpy.allclose(result, expected, atol=1e-2)
+                if passed:
+                    break
+                logging.warning("LDA failed to converge on attempt %i (got %s, expected %s)" %
+                                (i, result, expected))
+            self.assertTrue(passed)
+
+    def testPersistence(self):
+        model = ldamulticore.LdaMulticore(self.corpus, num_topics=2)
+        model.save(testfile())
+        model2 = ldamulticore.LdaMulticore.load(testfile())
+        self.assertEqual(model.num_topics, model2.num_topics)
+        self.assertTrue(numpy.allclose(model.expElogbeta, model2.expElogbeta))
+        tstvec = []
+        self.assertTrue(numpy.allclose(model[tstvec], model2[tstvec])) # try projecting an empty vector
+
+    def testLargeMmap(self):
+        model = ldamulticore.LdaMulticore(self.corpus, num_topics=2)
+
+        # simulate storing large arrays separately
+        model.save(testfile(), sep_limit=0)
+
+        model2 = ldamulticore.LdaMulticore.load(testfile())
+        self.assertEqual(model.num_topics, model2.num_topics)
+        self.assertTrue(numpy.allclose(model.expElogbeta, model2.expElogbeta))
+        tstvec = []
+        self.assertTrue(numpy.allclose(model[tstvec], model2[tstvec])) # try projecting an empty vector
+
+        # test loading the large model arrays with mmap
+        model2 = ldamulticore.LdaModel.load(testfile(), mmap='r')
+        self.assertEqual(model.num_topics, model2.num_topics)
+        self.assertTrue(numpy.allclose(model.expElogbeta, model2.expElogbeta))
+        tstvec = []
+        self.assertTrue(numpy.allclose(model[tstvec], model2[tstvec])) # try projecting an empty vector
+#endclass TestLdaMulticore
+
+
 class TestLdaMallet(unittest.TestCase):
     def setUp(self):
         self.corpus = mmcorpus.MmCorpus(datapath('testcorpus.mm'))
